// TODO: Add licence
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $URL:$
// $Id: $
// 
//
// Author(s)     : Michael Kerber <mkerber@mpi-inf.mpg.de>
//
// ============================================================================


#ifndef CGAL_INTERVAL_EVALUATE_1
#define CGAL_INTERVAL_EVALUATE_1  1

#include <iterator>

#include <CGAL/basic.h>
#include <boost/numeric/interval.hpp>

#include <CGAL/Coercion_traits.h>

CGAL_BEGIN_NAMESPACE

namespace internal {

 template<typename Polynomial_1, typename Bound, typename Coefficient>
 struct Interval_evaluate_1 : public std::binary_function
  <Polynomial_1,std::pair<Bound,Bound>,
  std::pair<typename CGAL::Coercion_traits<Coefficient,Bound>::Type,
  typename CGAL::Coercion_traits<Coefficient,Bound>::Type> > {
   
   typedef std::pair<typename CGAL::Coercion_traits<Coefficient,Bound>::Type,
     typename CGAL::Coercion_traits<Coefficient,Bound>::Type>
     result_type;

   typedef CGAL::Polynomial_traits_d<Polynomial_1> PT_1;
   
   result_type operator()(const Polynomial_1& p,
			  const std::pair<Bound,Bound>& b) 
     const {
     
     typename CGAL::Coercion_traits<Coefficient,Bound>::Cast cast;
     typedef typename CGAL::Coercion_traits<Coefficient,Bound>::Type 
       Coercion_type;
     typedef ::boost::numeric::interval<Coercion_type> Coercion_interval;
     typedef typename 
       PT_1::Coefficient_const_iterator Coefficient_const_iterator;
     
     Coercion_interval ix(cast(b.first),
			  cast(b.second));
     
     typedef typename PT_1::Coefficient_const_iterator_range 
       Coefficient_const_iterator_range;
     
     Coefficient_const_iterator_range range = 
       typename PT_1::Construct_coefficient_const_iterator_range()(p);
     
     Coefficient_const_iterator it = range.second - 1;
     
     Coercion_interval res(cast(*it));
     
     Coefficient_const_iterator p_begin = range.first;
     while((it--) != p_begin) 
       res = res * ix + Coercion_interval(cast(*it));
     return std::make_pair(res.lower(),res.upper());
   }
   
 };
 
} // namespace internal


CGAL_END_NAMESPACE

#endif // CGAL_INTERVAL_EVALUATE_1
