// Copyright (c) 2006-2008 Max-Planck-Institute Saarbruecken (Germany).
// All rights reserved.
//
// This file is part of CGAL (www.cgal.org); you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with CGAL.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Algebraic_foundations/include/CGAL/Test/_test_rational_traits.h $
// $Id: _test_rational_traits.h 47265 2008-12-08 06:26:27Z hemmer $
//
//
// Author(s)     : Michael Hemmer    <hemmer@mpi-inf.mpg.de>
//
// =============================================================================


// within this file FT ^= Fraction_traits<T>

#include <CGAL/basic.h>
#include <cassert>
#include <CGAL/to_rational.h>

#ifndef CGAL_TEST_RATIONAL_TRAITS_H
#define CGAL_TEST_RATIONAL_TRAITS_H

CGAL_BEGIN_NAMESPACE

template <class Rational>
void test_rational_traits(){
    Rational x = Rational(7)/Rational(2);
    
    typedef Rational_traits<Rational> Rational_traits;
    typedef typename Rational_traits::RT RT;
    BOOST_STATIC_ASSERT((::boost::is_same<RT,RT>::value));
     
    assert( Rational_traits().numerator(x) == RT(7));
    assert( Rational_traits().denominator(x) == RT(2));
    assert( Rational_traits().make_rational(RT(7),RT(2)) == x);
    assert( Rational_traits().make_rational(x,x) == Rational(1));

    // gloabal function to_rational 
    x = CGAL::to_rational<Rational>(3.5);
    assert( x == Rational(7)/Rational(2));
}

CGAL_END_NAMESPACE

#endif //  CGAL_TEST_RATIONAL_TRAITS_H
